﻿Imports System
Imports System.Linq
Imports Microsoft.EntityFrameworkCore
Imports Data_Points_0917_EFCore2Model.SamuraiApp.Domain

Namespace SamuraiApp.Data
    Public Class SamuraiContext
        Inherits DbContext

        Public Sub New(ByVal options As DbContextOptions(Of SamuraiContext))
            MyBase.New(options)
        End Sub

        Public Property Samurais As DbSet(Of Samurai)
        Public Property Quotes As DbSet(Of Quote)
        Protected Overrides Sub OnConfiguring(ByVal optionsBuilder As DbContextOptionsBuilder)
            optionsBuilder.UseInMemoryDatabase("onedb")
            MyBase.OnConfiguring(optionsBuilder)
        End Sub
        Protected Overrides Sub OnModelCreating(ByVal modelBuilder As ModelBuilder)

            'Note that we need to configure the model
            'to know about private entrance 
            'before running the code that depends on GetEntityTypes

            modelBuilder.Entity(Of Samurai).HasOne(GetType(Entrance), "Entrance").WithOne.HasForeignKey(GetType(Entrance), "SamuraiFk")

            For Each entityType In modelBuilder.Model.GetEntityTypes
                'LastModified is a shadow property, not props in the classes
                modelBuilder.Entity(entityType.Name).Property(Of Date)("LastModified")
                'IsDirty is for local tracking, not persisted in the database
                modelBuilder.Entity(entityType.Name).Ignore("IsDirty")
            Next entityType
            'NOTE: owned entity needs to go after the GetEntityTypes or it will be seen as an entity
            modelBuilder.Entity(Of Samurai)().OwnsOne(GetType(PersonFullName), "SecretIdentity")

        End Sub

        Public Overrides Function SaveChanges() As Integer
            For Each entryItem In ChangeTracker.Entries.Where(Function(e) e.State = EntityState.Added OrElse e.State = EntityState.Modified)
                'ignore owned entities (todo: is there a generic way?)
                If Not (TypeOf entryItem.Entity Is PersonFullName) Then
                    entryItem.Property("LastModified").CurrentValue = Date.Now
                End If
            Next entryItem

            Return MyBase.SaveChanges
        End Function


    End Class
End Namespace